package tdp024.todo.data.test.facade;

import java.util.ArrayList;
import java.util.List;
import org.junit.After;
import org.junit.Assert;
import org.junit.Test;
import tdp024.todo.data.api.entity.Todo;
import tdp024.todo.data.api.exception.TodoEntityNotFoundException;
import tdp024.todo.data.api.exception.TodoInputParameterException;
import tdp024.todo.data.api.facade.TodoEntityFacade;
import tdp024.todo.data.api.util.StorageFacade;
import tdp024.todo.data.db.facade.TodoEntityFacadeDB;
import tdp024.todo.data.db.util.StorageFacadeDB;
import tdp024.todo.data.flatfile.facade.TodoEntityFacadeFlatfile;
import tdp024.todo.data.flatfile.util.StorageFacadeFlatfile;

public class TodoEntityFacadeTest {

    //------------------ Units under test ------------------------------------//
    // We can easily swap between the DB and the flatfile implementations
    // ---- Please experiment with the different implementations ---- 
    private TodoEntityFacade todoEntityFacade = new TodoEntityFacadeDB();
    private StorageFacade storageFacade = new StorageFacadeDB();
    //------------------------------------------------------------------------//

    @After
    public void tearDown()  {
        storageFacade.emptyStorage();
    }
    
    /*
     * This is how we can test for exceptions.
     * We expect a TodoInputParameterException exception to happen,
     * if it doesn't the test will fail.
     */
    @Test(expected = TodoInputParameterException.class)
    public void testCreateException() throws Exception {
        todoEntityFacade.create(null, null);
    }
    
    /* 
     * We expect not to find a todo
     */
    @Test(expected = TodoEntityNotFoundException.class)
    public void testNotFoundException() throws Exception {
        todoEntityFacade.find(1000);
    }

    @Test
    public void testCreate() throws Exception {


        String title = "First thing to do";
        String body = "Brush your teeth";

        long id = todoEntityFacade.create(title, body);

        Todo todo = todoEntityFacade.find(id);

        Assert.assertEquals(title, todo.getTitle());
        Assert.assertEquals(body, todo.getBody());

    }

    @Test
    public void testActivate() throws Exception  {


        String title = "First thing to do";
        String body = "Brush your teeth";

        long id = todoEntityFacade.create(title, body);

        {
            Todo todo = todoEntityFacade.find(id);
            Assert.assertEquals(false, todo.isActive());
        }

        {
            todoEntityFacade.setActive(id, true);
            Todo todo = todoEntityFacade.find(id);
            Assert.assertEquals(true, todo.isActive());
        }
        {
            todoEntityFacade.setActive(id, false);
            Todo todo = todoEntityFacade.find(id);
            Assert.assertEquals(false, todo.isActive());
        }

    }


    @Test
    public void testFindAll() throws Exception {

        todoEntityFacade.create("First Title", "First Body");
        todoEntityFacade.create("Second Title", "Second Body");
        todoEntityFacade.create("Third Title", "Third Body");

        List<Todo> todos = todoEntityFacade.findAll();

        Assert.assertEquals(3, todos.size());
        
        List<String> titles = new ArrayList<String>();
        titles.add(todos.get(0).getTitle());
        titles.add(todos.get(1).getTitle());
        titles.add(todos.get(2).getTitle());
        
        Assert.assertTrue(titles.contains("First Title"));
        Assert.assertTrue(titles.contains("Second Title"));
        Assert.assertTrue(titles.contains("Third Title"));

    }
}