package tdp024.todo.logic.impl.facade;

import tdp024.todo.data.api.entity.Todo;
import tdp024.todo.data.api.exception.TodoEntityNotFoundException;
import tdp024.todo.data.api.exception.TodoInputParameterException;
import tdp024.todo.data.api.exception.TodoServiceConfigurationException;
import tdp024.todo.data.api.facade.TodoEntityFacade;
import tdp024.todo.logic.api.facade.TodoLogicFacade;
import tdp024.util.logger.TodoLogger;
import tdp024.util.logger.TodoLoggerImpl;

public class TodoLogicFacadeImpl implements TodoLogicFacade {

    private static final TodoLogger todoLogger = new TodoLoggerImpl();
    private TodoEntityFacade todoEntityFacade;

    public TodoLogicFacadeImpl(TodoEntityFacade todoEntityFacade) {
        this.todoEntityFacade = todoEntityFacade;
    }

    /*
     * The logic layer can not do much about the input being 
     * incorrect, this will be directly thrown back to the calling layer
     * (in our case the web layer).
     * 
     * However service errors might be important, we want to log these, and
     * then rethrow the same exception
     * 
     */
    @Override
    public long create(
            String title,
            String body)
            throws
            TodoInputParameterException,
            TodoServiceConfigurationException {
        try {
            return todoEntityFacade.create(title, body);
        } catch (TodoServiceConfigurationException e) {
            todoLogger.log(e);
            throw e;
        }

    }

    @Override
    public Todo find(long id)
            throws
            TodoEntityNotFoundException,
            TodoServiceConfigurationException {
        try {
            return todoEntityFacade.find(id);
        } catch (TodoServiceConfigurationException e) {
            todoLogger.log(e);
            throw e;
        }
    }

    @Override
    public void checkOut(long id)
            throws
            TodoEntityNotFoundException,
            TodoServiceConfigurationException {
        try {
            this.todoEntityFacade.setActive(id, true);
        } catch (TodoServiceConfigurationException e) {
            todoLogger.log(e);
            throw e;
        }
    }

    @Override
    public void checkIn(long id)
            throws
            TodoEntityNotFoundException,
            TodoServiceConfigurationException {
        try {
            this.todoEntityFacade.setActive(id, false);
        } catch (TodoServiceConfigurationException e) {
            todoLogger.log(e);
            throw e;
        }
    }
}
